///////////////////////////////////////////////////////////////////////////////
// Camera.h
// ========
// OpenGL camera class
//
//  AUTHOR: Song Ho Ahn (song.ahn@gmail.com)
// CREATED: 2011-12-02
// UPDATED: 2012-03-23
///////////////////////////////////////////////////////////////////////////////

#ifndef GIL_CAMERA_H
#define GIL_CAMERA_H

#include "Vectors.h"
#include "Matrices.h"
#include "animUtils.h"
#include "Quaternion.h"

namespace Gil
{


class Camera
{
public:
    Camera();
    ~Camera();

    void update(float frameTime);
    void printSelf();

    void lookAt(const Vector3& target);
    void lookAt(float x, float y, float z)          { lookAt(Vector3(x,y,z)); }
    void lookAt(const Vector3& target, const Vector3& up);
    void moveTo(const Vector3& to, float duration=0, AnimationMode mode=EASE_IN_OUT);
    void rotateTo(float angle, const Vector3& axis, float duration=0, AnimationMode mode=EASE_IN_OUT);

    // getters/setters
    void setPosition(const Vector3& v);
    void setPosition(float x, float y, float z)     { setPosition(Vector3(x,y,z)); }
    void setRotation(const Vector3& v); // angles in degree
    void setRotation(float x, float y, float z)     { setRotation(Vector3(x,y,z)); }
    void setRotation(const Quaternion& q);
    void setQuaternion(const Quaternion& q);

    const Vector3& getPosition() const  { return position; }
    const Vector3& getRotation() const  { return rotation; }
    const Matrix4& getMatrix() const    { return matrix; }
    Vector3 getLeftAxis() const;
    Vector3 getUpAxis() const;
    Vector3 getForwardAxis() const;

protected:

private:
    void updateMove(float frameTime);
    void updateTurn(float frameTime);

    //member variables
    Vector3 position;           // x,y,z
    Vector3 rotation;           // x,y,z angles in degree
    Vector3 target;             // x,y,z lookat position
    Matrix4 matrix;             // 4x4 matrix
    Quaternion quaternion;      // quaternion for rotations

    Vector3 movingFrom;
    Vector3 movingTo;
    Vector3 movingVector;       // normalized direction
    float   movingTime;
    float   movingDuration;
    float   movingDistance;     // length of movement
    bool    moving;
    AnimationMode movingMode;

    Vector3 turningFrom;
    Vector3 turningTo;
    Vector3 turningVector;      // normalized
    float   turningTime;
    float   turningDuration;
    float   turningDistance;    // length of rotation
    bool    turning;
    AnimationMode turningMode;


};


} // namespace
#endif
